{% if aux.get("pip", {}).get("entry_points_filecontent", "") -%}
# remove entry_points before install
RUN DIST_INFO_DIR="$(python -c 'import site; print(site.getusersitepackages())')/{{aux.project.name}}-{{aux.python_project.version}}.dist-info" \
    /bin/sh -c 'rm -rf $DIST_INFO_DIR'
{% endif %}
